@@ -5,11 +5,11 @@ module Doorkeeper
     class BaseRequest
       include Validations
 
-      attr_reader :grant_type
+      attr_reader :grant_type, :server
+
+      delegate :default_scopes, to: :server
 
       def authorize
-        validate
-
         if valid?
           before_successful_response
           @response = TokenResponse.new(access_token)
@@ -26,22 +26,14 @@ module Doorkeeper
         @scopes ||= build_scopes
       end
 
-      def default_scopes
-        server.default_scopes
-      end
-
-      def valid?
-        error.nil?
-      end
-
-      def find_or_create_access_token(client, resource_owner_id, scopes, server)
-        context = Authorization::Token.build_context(client, grant_type, scopes)
+      def find_or_create_access_token(client, resource_owner, scopes, server)
+        context = Authorization::Token.build_context(client, grant_type, scopes, resource_owner)
         @access_token = server_config.access_token_model.find_or_create_for(
-          client,
-          resource_owner_id,
-          scopes,
-          Authorization::Token.access_token_expires_in(server, context),
-          Authorization::Token.refresh_token_enabled?(server, context),
+          application: client,
+          resource_owner: resource_owner,
+          scopes: scopes,
+          expires_in: Authorization::Token.access_token_expires_in(server, context),
+          use_refresh_token: Authorization::Token.refresh_token_enabled?(server, context),
         )
       end
 
@@ -63,10 +55,10 @@ module Doorkeeper
         if @original_scopes.present?
           OAuth::Scopes.from_string(@original_scopes)
         else
-          client_scopes = @client.try(:scopes)
+          client_scopes = @client&.scopes
           return default_scopes if client_scopes.blank?
 
-          default_scopes & @client.scopes
+          default_scopes & client_scopes
         end
       end
     end
