@@ -2,9 +2,9 @@
 
 module Doorkeeper
   module OAuth
-    class ClientCredentialsRequest < BaseRequest
+    module ClientCredentials
       class Issuer
-        attr_accessor :token, :validator, :error
+        attr_reader :token, :validator, :error
 
         def initialize(server, validator)
           @server = server
@@ -19,6 +19,7 @@ module Doorkeeper
             @token = false
             @error = validator.error
           end
+
           @token
         end
 
@@ -29,6 +30,7 @@ module Doorkeeper
             client,
             Doorkeeper::OAuth::CLIENT_CREDENTIALS,
             scopes,
+            nil,
           )
           ttl = Authorization::Token.access_token_expires_in(@server, context)
 
