@@ -3,18 +3,12 @@
 module Doorkeeper
   module OAuth
     class ClientCredentialsRequest < BaseRequest
-      attr_accessor :server, :client, :original_scopes
-      attr_reader :response
-      attr_writer :issuer
+      attr_reader :client, :original_scopes, :response
 
       alias error_response response
 
       delegate :error, to: :issuer
 
-      def issuer
-        @issuer ||= Issuer.new(server, Validator.new(server, self))
-      end
-
       def initialize(server, client, parameters = {})
         @client = client
         @server = server
@@ -26,6 +20,13 @@ module Doorkeeper
         issuer.token
       end
 
+      def issuer
+        @issuer ||= ClientCredentials::Issuer.new(
+          server,
+          ClientCredentials::Validator.new(server, self),
+        )
+      end
+
       private
 
       def valid?
