@@ -7,10 +7,10 @@ module Doorkeeper
 
       def request
         @request ||= OAuth::PasswordAccessTokenRequest.new(
-          Doorkeeper.configuration,
+          Doorkeeper.config,
           client,
           resource_owner,
-          parameters
+          parameters,
         )
       end
     end
