@@ -1,14 +1,10 @@
 # frozen_string_literal: true
 
 module Doorkeeper
-  def self.gem_version
-    Gem::Version.new VERSION::STRING
-  end
-
   module VERSION
     # Semantic versioning
     MAJOR = 5
-    MINOR = 3
+    MINOR = 5
     TINY = 0
     PRE = nil
 
